/*
Rappaport, Jordan. 2018. Stata replication code for the paper 
"The Faster Growth of Larger, Less Crowded Locations". 
Federal Reserve Bank of Kansas City Economic Review 103, no. 4. 
Available at https://www.kansascityfed.org/people/jordanrappaport 

Thank you to Anissa Khan for excellent research assistance.

Descriptions of variables are included in the file, "rappaport_frbkc_er_2018_data_and_description.xlsx"

To run, stata must be open to the directory in which the data file, 
"rappaport_frbkc-er_2018_data.dta", and the ado file, "reg_conley.ado" 
are located. To do so, use the "cd" command.  
*/



program drop _all
clear
use  rappaport_frbkc-er_2018_data.dta /* make sure data file is in the directory to which stata is open. you can check using "pwd" command */
describe, short 



* *****************************************************************************************
* vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv variable construction vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv


*  ************* spline of log population
generate lpop_2000_00_08     = min(max(00,lpop_2000),08)
generate lpop_2000_08_09     = min(max(08,lpop_2000),09) - (08)  
generate lpop_2000_09_10     = min(max(09,lpop_2000),10) - (09)  
generate lpop_2000_10_11     = min(max(10,lpop_2000),11) - (10)  
generate lpop_2000_11_12     = min(max(11,lpop_2000),12) - (11)  
generate lpop_2000_12_13     = min(max(12,lpop_2000),13) - (12)  
generate lpop_2000_13_14     = min(max(13,lpop_2000),14) - (13)
generate lpop_2000_14_99     = min(max(14,lpop_2000),99) - (14)  

*  ************* spline of log mean pop density
generate lppm2mn_2000_00_03  = min(max(00.0,lpop_mi2_tract_2000_mn),03.0)
generate lppm2mn_2000_03_04  = min(max(03.0,lpop_mi2_tract_2000_mn),04.0) - (03.0)
generate lppm2mn_2000_04_05  = min(max(04.0,lpop_mi2_tract_2000_mn),05.0) - (04.0)  
generate lppm2mn_2000_05_06  = min(max(05.0,lpop_mi2_tract_2000_mn),06.0) - (05.0)  
generate lppm2mn_2000_06_07  = min(max(06.0,lpop_mi2_tract_2000_mn),07.0) - (06.0)  
generate lppm2mn_2000_07_08  = min(max(07.0,lpop_mi2_tract_2000_mn),08.0) - (07.0)  
generate lppm2mn_2000_08_99  = min(max(08.0,lpop_mi2_tract_2000_mn),99.9) - (08.0)  


*  ************* log of percentile measures of pop density
generate lpop_mi2_tract_2000_p25      =  log(pop_mi2_tract_2000_p25) 
generate lpop_mi2_tract_2000_p75      =  log(pop_mi2_tract_2000_p75) 
generate lpop_mi2_tract_2000_p95      =  log(pop_mi2_tract_2000_p95) 
generate lpop_mi2_tract_2000_p99      =  log(pop_mi2_tract_2000_p99)

*  ************* change in log pop density between percentiles (see discussion in endnote 25)
generate dlpop_mi2_tract_2000_p25p75  =  lpop_mi2_tract_2000_p75 - lpop_mi2_tract_2000_p25
generate dlpop_mi2_tract_2000_p75p95  =  lpop_mi2_tract_2000_p95 - lpop_mi2_tract_2000_p75
generate dlpop_mi2_tract_2000_p95p99  =  lpop_mi2_tract_2000_p99 - lpop_mi2_tract_2000_p95




* *****************************************************************************************
* vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv lists of variables vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv

* Define stata macros for sets of right-hand-side variables  (can be referenced preceded by a dollar sign)
/* 

Descriptions of variables are included in the file, "rappaport_frbkc_er_2018_data_and_description.xlsx".

The weather and hilliness characteristics includes both a linear variable and a quadratic variable.
The quadratic variables are constructed subtracting the sample mean of the characteristic prior to
squaring. Doing so does not affect the estimated coefficients on the quadratic variables but does
affect the estimated coefficients on the linear variables, which now correspond to the partial 
derivative of growth with respect to the rhs characteristic evaluated at its sample mean. 


*/


global pop_spline            "lpop_2000_00_08 lpop_2000_08_09 lpop_2000_09_10 lpop_2000_10_11 lpop_2000_11_12 lpop_2000_12_13 lpop_2000_13_14 lpop_2000_14_99"

global mean_density_spline   "lppm2mn_2000_00_03 lppm2mn_2000_03_04 lppm2mn_2000_04_05 lppm2mn_2000_05_06 lppm2mn_2000_06_07 lppm2mn_2000_07_08 lppm2mn_2000_08_99"

global metro_adjacency       "i_adj_metro_larg i_adj_metro_small"  /* 2 vars */
global weather               "wthr_temp_max_01 wthr_temp_max_01_dmn2 wthr_hindex_max_07 wthr_hindex_max_07_dmn2 wthr_relhumid_mean_07 wthr_relhumid_mean_07_dmn2 wthr_rain_13 wthr_rain_13_dmn2 wthr_raindays_13 wthr_raindays_13_dmn2" /* 10 vars */
global coast_river           "i_wtr_can i_wtr_cas i_wtr_cg i_wtr_cp i_wtr_cl i_wtr_rn i_wtr_rm" /* 7 vars */
global hilliness             "altsd_to_area altsd_to_area_dmn2"  /* 2 vars */
global shale                 "i_shale_anadarko i_shale_bakken i_shale_eagle i_shale_haynesville i_shale_niobrara i_shale_permian" /* 6 vars */
global higher_ed             "enrl_13up_shr_pop_2000" /* 1 var */
global industry_comp         "lci_sc_ag_2000 lci_sc_mn_2000 lci_sc_mnf_2000 lci_sc_whl_2000 lci_sc_rtl_2000 lci_sc_trn_whr_2000 lci_sc_utl_2000 lci_sc_info_2000 lci_sc_fin_insr_2000 lci_sc_rest_2000 lci_sc_prf_sci_tch_2000 lci_sc_mng_2000 lci_sc_admn_wst_2000 lci_sc_edu_2000 lci_sc_hlth_scl_2000 lci_sc_art_ent_rec_2000 lci_sc_accm_fd_2000 lci_sc_srv_msc_2000" /* 18 vars */
global occupation_comp       "lco_sc_mng_bus_fin_2000 lco_sc_cmp_mth_2000 lco_sc_arch_eng_2000 lco_sc_sci_2000 lco_sc_lgl_2000 lco_sc_edu_2000 lco_sc_art_ent_2000 lco_sc_hlth_2000 lco_sc_hlthsup_2000 lco_sc_fd_2000 lco_sc_srv_bldg_2000 lco_sc_prscr_2000 lco_sc_sls_2000 lco_sc_admn_2000 lco_sc_frm_2000 lco_sc_extr_2000 lco_sc_instl_mnt_rpr_2000 lco_sc_prd_2000 lco_sc_trn_2000 lco_sc_rl_wtr_othr_2000 lco_sc_trn_mtrlmv_2000" /* 21 vars */

global baseline              "$metro_adjacency $weather $coast_river $hilliness $shale $higher_ed"



*  Calculate R-squared values regressing growth on control variables only:
* (all regressions drop smallest observation, population 67, as a potential outlier; next smallest has population 356)

* baseline vars:
regress g17pct_pop_2000 $baseline if pop_2000 >= 100  

* baseline vars + industry composition
regress g17pct_pop_2000 $baseline $industry_comp if pop_2000 >= 100  

* baseline vars + industry composition + occupation composition
regress g17pct_pop_2000 $baseline $industry_comp $occupation_comp if pop_2000 >= 100  



* *****************************************************************************************
* vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv regressions vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv

/*
Regressions in Tables 1, 2, and 3 estimate standard errors robust to stochastic disturbances
that are spatially correlated across observations. These are implemented with a stata command,
"reg_conley", which is included in the file "reg_conley.ado". With stata pointed to a directory 
in which that file is located, the command will be automatically loaded the first time it is 
executed. The file includes extensive documentation of the command syntax and
the econometric theory on which it is based. 
*/

* All regressions drop smallest observation, population 67, as a potential outlier. Next smallest observation has population 356.


*  ************* Table 1 regressions 
* Table 1 Column 1, pop density spline only
reg_conley g17pct_pop_2000 $pop_spline , if(pop_2000 >= 100) crdlst(geo_lat_2000 geo_lng_2000) cut(500) 

* Table 1 Column 2, pop density spline + baseline 
reg_conley g17pct_pop_2000 $pop_spline : $baseline  , if(pop_2000 >= 100) crdlst(geo_lat_2000 geo_lng_2000) cut(500) 

* Table 1 Column 3, pop density spline + baseline + industry composition
reg_conley g17pct_pop_2000 $pop_spline : $baseline $industry_comp , if(pop_2000 >= 100) crdlst(geo_lat_2000 geo_lng_2000) cut(500) 

* Table 1 Column 4, pop density spline + baseline + industry composition + occupation composition
reg_conley g17pct_pop_2000 $pop_spline : $baseline $industry_comp $occupation_comp, if(pop_2000 >= 100) crdlst(geo_lat_2000 geo_lng_2000) cut(500) 

* NOTE: Syntax above suppresses reporting of coefficients on variables to the right of the colon. 
*       To report all coefficients, simply delete the colon. Doing so will increase computation time.



*  ************* Table 2 regressions 
* Table 2 Column 1, mean density spline
reg_conley g17pct_pop_2000 $mean_density_spline , if(pop_2000 >= 100) crdlst(geo_lat_2000 geo_lng_2000) cut(500) 

* Table 2 Column 2, mean density spline + baseline 
reg_conley g17pct_pop_2000 $mean_density_spline : $baseline  , if(pop_2000 >= 100) crdlst(geo_lat_2000 geo_lng_2000) cut(500) 

* Table 2 Column 3, mean density spline + baseline + industry composition
reg_conley g17pct_pop_2000 $mean_density_spline : $baseline $industry_comp , if(pop_2000 >= 100) crdlst(geo_lat_2000 geo_lng_2000) cut(500) 

* Table 2 Column 4, mean density spline + baseline + industry composition + occupation composition
reg_conley g17pct_pop_2000 $mean_density_spline : $baseline $industry_comp $occupation_comp, if(pop_2000 >= 100) crdlst(geo_lat_2000 geo_lng_2000) cut(500) 

* NOTE: Syntax above suppresses reporting of coefficients on variables to the right of the colon.
*       To report all coefficients, simply delete the colon. Doing so will increase computation time.



*  ************* Table 3 regressions: 

/* 
Note: the conley procedure for estimating standard errors that are robust to spatially-correlated stochastic disturbances
is asympotically consistent. But it may be a poor estimator of standard errors with a limited number of observations, such
as the 87 medium and large metropolitan areas used in the regressions reported in Table 3.  Counterintuitively, most of the 
OLS standard errors for the coefficients reported in Table 3 exceed the reported Conley standard errors. However, all coefficients
starred as statistically significant in Table 3 are also statistically significant at the same benchmark level using the OLS
standard errors
*/

* Table 3 Column 1
reg_conley g17pct_pop_2000            lpop_mi2_tract_2000_p95                              , if(pop_2000 >= 500000) crdlst(geo_lat_2000 geo_lng_2000) cut(500)  /* implicitly sptlcoef(1) */

* Table 3 Column 2
reg_conley g17pct_pop_2000 lpop_2000                                                       , if(pop_2000 >= 500000) crdlst(geo_lat_2000 geo_lng_2000) cut(500) /* implicitly sptlcoef(1) */

* Table 3 Column 3                                                                                            
reg_conley g17pct_pop_2000 lpop_2000  lpop_mi2_tract_2000_p95                              , if(pop_2000 >= 500000) crdlst(geo_lat_2000 geo_lng_2000) cut(500) /* implicitly sptlcoef(1) */

* Table 3 Column 4                                                                                            
reg_conley g17pct_pop_2000                                    dlpop_mi2_tract_2000_p95p99  , if(pop_2000 >= 500000) crdlst(geo_lat_2000 geo_lng_2000) cut(500) /* implicitly sptlcoef(1) */  

* Table 3 Column 5
reg_conley  g17pct_pop_2000 lpop_2000 lpop_mi2_tract_2000_p95 dlpop_mi2_tract_2000_p95p99  , if(pop_2000 >= 500000) crdlst(geo_lat_2000 geo_lng_2000) cut(500) /* implicitly sptlcoef(1) */

* NOTE: Syntax above suppresses reporting of coefficients on variables to the right of the colon. 
*       To report all coefficients, simply delete the colon. Doing so will increase computation time.


*  *** Table 3, supplemental regressions (see endnote 23)

 * additionally including 25th and 75th pctile density boosts R-sqrd from Table 3 column 5 regression to 0.38
 reg_conley  g17pct_pop_2000 lpop_2000  lpop_mi2_tract_2000_p25      lpop_mi2_tract_2000_p75      lpop_mi2_tract_2000_p95  dlpop_mi2_tract_2000_p95p99  , if(pop_2000 >= 500000) crdlst(geo_lat_2000 geo_lng_2000) cut(500) /* implicitly sptlcoef(1) */
                                                                                                                           
 * equivalent specification, which is easier to interpret, of the regression immediately above. 
 reg_conley  g17pct_pop_2000 lpop_2000  lpop_mi2_tract_2000_p25  dlpop_mi2_tract_2000_p25p75  dlpop_mi2_tract_2000_p75p95  dlpop_mi2_tract_2000_p95p99 , if(pop_2000 >= 500000) crdlst(geo_lat_2000 geo_lng_2000) cut(500) /* implicitly sptlcoef(1) */


 * 25th, 75th, 95th, and 99th pctile densities give R-sqrd of 0.37 (equation two above sans initial population)
 reg_conley  g17pct_pop_2000            lpop_mi2_tract_2000_p25      lpop_mi2_tract_2000_p75      lpop_mi2_tract_2000_p95      lpop_mi2_tract_2000_p99 , if(pop_2000 >= 500000) crdlst(geo_lat_2000 geo_lng_2000) cut(500) /* implicitly sptlcoef(1) */
                                                                                                                           
 * equivalent specification, which is easier to interpret, of regression immediately above.
 reg_conley  g17pct_pop_2000            lpop_mi2_tract_2000_p25  dlpop_mi2_tract_2000_p25p75  dlpop_mi2_tract_2000_p75p95  dlpop_mi2_tract_2000_p95p99 , if(pop_2000 >= 500000) crdlst(geo_lat_2000 geo_lng_2000) cut(500) /* implicitly sptlcoef(1) */
                                                                                                                           

* NOTE: Syntax above suppresses reporting of coefficients on variables to the right of the colon. 
*       To report all coefficients, simply delete the colon. Doing so will increase computation time.


* *****************************************************************************************
* vvvvvvvvvvvvvvvvvvvvvvvvvv constructing fitted values in charts vvvvvvvvvvvvvvvvvvvvvvvvvv

/* Charts 1, 2, 3
x-axis: lpop_2000 (ln(population 2000))
y-axis: g17pct_pop_2000 (avg. annual population growth, 2000-2017)

The command below constructs a variable, "fit_g17pct_lpop2000_deg1_bw075", equal to the fitted values
from a kernel regression of g17pct_pop_2000 on lpop_2000 using a linear polynomial with epanechnikov 
weighting and a bandwidth of 0.75 log points
*/
lpoly g17pct_pop_2000 lpop_2000 if pop_2000 > 100, kernel(epan) degree(1) bwidth(0.75) at(lpop_2000) generate(fit_g17pct_lpop2000_deg1_bw075)


/*Chart 6
x-axis: lemp_sf3_2000    (persons reporting that they worked for pay, 2000 decennial census)
y-axis: g14pct_emp_acs_2000 (avg. annual employment growth 2000-2014)

The command below constructs a variable, "fit_g14pct_lemp2000_deg1_bw075", equal to the fitted values
from a kernel regression of g14pct_emp_acs_2000 on lemp_sf3_2000 using a linear polynomial with epanechnikov
weighting and a bandwidth of 0.75 log points
*/ 
lpoly g14pct_emp_acs_2000 lemp_sf3_2000 if pop_2000 > 100, kernel(epan) degree(1) bwidth(0.75) at(lemp_sf3_2000) generate(fit_g14pct_lemp2000_deg1_bw075) 


/*Chart 7
x-axis: lpop_mi2_tract_2000_p95 
y-axis: g17pct_pop_2000

The command below constructs a variable, "fit_g17pct_lpopmi2p95", equal to the fitted values
from an OLS regression of g17pct_pop_2000 on lpop_mi2_tract_2000_p95
*/
regress g17pct_pop_2000 lpop_mi2_tract_2000_p95
predict fit_g17pct_lpopmi2p95, xb   




