*  Stata code to estimate standard errors robust to spatial correlation of stochastic component across observations. 
*  Generalizes Huber-White estimator of standard errors robust to heteroscedastic stochastic component ('robust' 
*  option in Stata) and the variation of Huber-White that relaxes independence assumption among groups of observations
*  ('cluster' option in Stata). 

*! version 2019_03_19a  (functionally identical to 2019_02_13a and 2019_02_07b)


* ^vvvvvvvvvvvvvvvvvvvvvvvv documentation vvvvvvvvvvvvvvvvvvvvvvvv {

*  SUGGESTED CITATION: 
*  Rappaport, Jordan. 2019. Stata Code for OLS Estimation with Spatially Correlated Disturbances. 
*  Available at https://kansascityfed.org/people/jordanrappaport 

*  SYNTAX:
*  reg_conley lhs_var rhs_variables_to_report : rhs_variables_not_reported , crdlst(lattitude_variable longitude_variable) cut(distance_in_km)			
*  [if(condition for observation being included in regression)] [obsweight(weighting_variable)] 
*  [sptlcoef(strictly positive scalar describing decline convexity of spatial correlation) suppress] 
* 
*  Does not allow for "hascons"  or "nocons" options. 
*  Does not allow standard syntax for stata weighiting options ("aweight fweight pweight iweight"),
*  but does allow for aweights using syntax described below.
* 
*  'rhs_variables_to_report : rhs_variables_not_reported': The colon and second set of variables are optional.  
*       Computation time is proportional to product of the number of rhs variables and the number of observations
*       To save time, robust spatial standard errors can be computed only for a subset of the rhs variables. 
*       Using this optional syntax, the program will report coefficients and standard errors only for this subset.
*       Reported regression statistics (e.g., R-squared) describe the regression on all of the rhs variables. 
*  crdlst    (required): name of a variable giving lattitude followed by name of variable giving longitude (in degrees)
*  cut       (required): this is the distance threshold (in km) above which the covariance of the stochastic 
*                        component is assumed to be zero ['D' in formulas below]                              
*  if        (optional): observations must satisfy this condition to be included in the regression
*  obsweight (optional): equivalent to stata aweight option; defaults to equal weighting for each observation;
*                        a rationale for using weights when observations differ in population is discussed in Rappaport (2007) 
*  sptlcoef  (optional): defaults to 1 (linear decline) ['c' in forumlas below]. If set >= 99, spatial weighting is non-continuous:
*                        1 if distace between observations <= cut distance and 0 otherwise.
*  suppress  (optional): suppresses display of table comparing alternative standard errors (OLS, White, Conley)
* 
*  After using 'reg_conley' command, all the standard OLS 'regress' procedure matrices will be in memory.
*  There will also be a matrix `cov_dep', the variance covariance matrix estimated to allow for spatial correlation.						
* 
*  Procedure will fail if any right-hand-side variables for which coefficients are to be displayed (i.e., to left of colon)
*  need to be dropped from the regression (e.g., due to colinearity or lack of variation). If this happens, you can run Stata's
*  standard "regress" command to see which variables need to be dropped. Then run "reg_conley" explicitly excluding the variables
*  dropped by regress. 
* 
* 
*  BACKGROUND:                                                            				                                                               									                                                                                   
* 	Estimator is based on research in:													                                                                                                      
* 
* 	1. Conley, Timothy G. 1999. "GMM Estimation with Cross Sectional Dependence."	                                               
* 	   Journal of Econometrics vol. 92, no. 1: 1-45. Available at https://doi.org/10.1016/S0304-4076(98)00084-0
* 
*   2. Conley, Timothy G. 2008. "Spatial Econometrics." The New Palgrave Dictionary of Economics,
*      2nd Edition. Steven N. Durlauf and Lawrence E. Blume, eds., Palgrave Macmillan. Available at https://doi.org/10.1057/978-1-349-95121-5_2023-1
* 
*   3. Dube, Jean-Pierre. 1997. Stata v5.0 Code for OLS Estimation for X-Sectional Data with Location-Based Dependence.
* 														                                                                                                     
* 	4. Rappaport, Jordan. 2007. "Moving to Nice Weather." Regional Science and                                                    
*      Urban Economics, vol. 37, no. 3: 375-398. Available at https://doi.org/10.1016/j.regsciurbeco.2006.11.004										                                                                 


*  A revised version of Dube(1997) that does not substantively differ, Stata v6.0 Code for OLS Estimation with Location-Based Dependence,
*  is available at http://economics.uwo.ca/people/conley_docs/code_to_download_gmm.html

*  Dube's website also includes stata code for instrumental variable estimation with location-based dependence.
*  Solomon Hsiang has posted code that implements the Conley adjustment for panel data ('ols_spatial_HAC.ado')
* 	
*  I implement estimator using two Stata programs: 'reg_conley' formats input, calls program 'x_ols_modified', and then formats output.
*  'x_ols_modified' includes code from Dube (1997). (Stata 5 version, October 11, 1997).
* 
* 
*  Spatial decline function is based on Euclidean distance, calculated using a great circle approximation between geographic points 
*  described by lattitude and longitude. An optional parameter specifies the convexity of the spatial decline function and regressions
*  can optonally be weighted. Rappaport (2007) discusses this convexity and the rationale for weighting when observations are locations
*  with differing population. The econometric theory described in Conley (1999) and the implementation in Dube (1997) use linear decline
*  based on the product of displacement in each of two dimensions. Conley (2008) discusses usng a step spatial decline: 1 out to some
*  cutoff distance and 0 beyond it. I include this step decline as an option, described below.
     


*  SPATIAL DECLINE FUNCTION:
*  Two scalar parameters, D>0 and c>0, govern the estimation of the covariance between the stochastic disturbances of any pair of observations, i and j. 
*  Let d_ij give the distance between the two. The spatial weighting, w_ij, is assumed to decline with convexity c from 1 when d_ij = 0 to 0 when d_ij 
*  equals or exceeeds threshold distance, D. More specifically:
*                              w_ij = 1 - (d_ij/D)^c             if d_ij<=D ; 0 for d_ij>D
*       partial(w_ij)/partial(d_ij) = -     (c)/(D^c)*d_ij^(c-1) if d_ij< D ; 0 for d_ij>D; negative for d_ij in (0,D) 
*   partial^2(w_ij)/partial(d_ij)^2 = -(c-1)(c)/(D^c)*d_ij^(c-2) if d_ij< D ; 0 for d_ij>D
*                                   >0 : c<1 & d_ij<D            [convex decline ... rapid  as d_ij increases from 0 and then slight as d_ij approaches D] 
*                                    0 : c=1 & d_ij<D            [linear decline]
*                                   <0 : c>1 & d_ij<D            [concave decline... slight as d_ij increases from 0 and then rapid  as d_ij approaches D]
* 
*   !!IMPLEMENTATION NOTE!!: if c is set >=99 ---> w_ij = 1 if d_ij<=D and 0 otherwise


* ^^^^^^^^^^^^^^^^^^^^^^^^^ documentation ^^^^^^^^^^^^^^^^^^^^^^^^ }



* vvvvvvvvvvvvvvvvvvvvvvvvv 'reg_conley' vvvvvvvvvvvvvvvvvvvvvvvvv {
*! "outer" program

capture program drop reg_conley
program define reg_conley /*syntax described above*/
version 5.0   /* Stata compatability... **!! NOTE: LOCAL MACROS IN VERSION 5 CAN BE AT MOST 7 CHARACTERS !!** */

preserve            /* routine creates a variable "_intrcpt" "_wght" */

local method "GLS with standard errors robust to spatially-correlated errors" /* For display purposes only*/
local frstcf "6"    /* Row on which first coefficient is printed */ 
local adjust "11"   /* The number of additional rows in display matrix:  i.e. for headings and summary statistics */ 
local ncntmax "10" /* Maximum number of control variables to display */


local input "`*'"  
di "`input'" 

parse "`*'", parse(" :,")   /* legacy version 4 command: splits string into tokens based on the parsing characters between quotations in option  */

local depvar "`1'"
mac shift           /* shift right along input arguments: former token 2 becomes token 1, former token 3 becomes token 2, ....*/

* put all rhs variables for which we want coefficients reported (to left of optional colon) into local macro `mainlst'
local mainlst ""
while ("`1'"!=":" & "`1'"!="," & "`1'"!="") {
	local mainlst "`mainlst' `1'"
	mac shift}                     /* puts all rhs variables for which we want coefficients reported into local macro, `mainlst' */
if ("`1'" == ":") { mac shift }  /*  shift past colon (if there is one) */

local rmndr "`*'"                      /* local macro rmndr is everything to right of ":" if there is a colon OR ELSE any conditioning and options following a comma */
local varlist "required existing"      /* I think that macro, `varlist', is used by the parse command when subsequently called*/
parse "`mainlst'"		                   /* puts variable names from `mainlst' into `varlist' */
local mainlst "`varlist'"              /* puts variable names back into `mainlst'           */
local options "sptlcoef(real 1) cut(real 200)  crdlst(string)  obsweight(string) if(string) SUPpress"  /*    nocons and hascons not allowed; possible future upgrade */

local varlist "optional existing none"
parse "`rmndr'"                        /* puts variable names in `rmndr' into `varlist' */
local cntrlst "`varlist'"              /* puts variable names into `cntrlst'*/
local nmain : word count `mainlst'  
local ncntrl : word count `cntrlst' 


*  Are there optional control variables (listed to right of ":") for which we do not want to report coefficients?
if("`cntrlst'" ~= ""){
      local stage1 "yes"
      local args_wc = `nmain'}      
else {
      local stage1 "no"
      local args_wc = `nmain' + 1}

di ""
di "dependent var:  `depvar'"
if `nmain'==1{
di "mainlst:        `nmain' var:  `mainlst'"}
else{
di "mainlst:        `nmain' vars: `mainlst'"
}

if ("`stage1'"=="yes" & `ncntrl'<=`ncntmax'){
    if `ncntrl'==1{
    di "cntrlst:        `ncntrl' var:  `cntrlst'"}
    else{
    di "cntrlst:        `ncntrl' vars: `cntrlst'"}    
    }
else if ("`stage1'"=="yes"){
    di "cntrlst:        `ncntrl' vars"}
else{
    di "cntrlst:        none"}
    
di "weight:         `obsweight'"
di "coords:         `crdlst'"
di "sptlcoef:       `sptlcoef'" 
di "cut dist:       `cut'" 
di "if:             `if'"
di ""

* CREATE CONSTANT VARIABLE
capture drop _intrcpt
generate     _intrcpt = 1

* IF NO WEIGHT SPECIFIED, WEIGHT EACH OBS BY 1
if ("`obsweight'" == ""){
   capture drop     _wght
   generate double  _wght = 1
   local oweight   "_wght"}
else{local oweight   "`obsweight'"}


* ** DEALING WITH MISSING OBSERVATIONS AND OPTIONAL IF CONDITION
*    set tempvar `touse'=1 iff there are no missing values, including for lat and long variables, AND optional if condition is met; 
*    'x_ols_modified' will only run for obs where `touse'==1
tempvar touse
quietly{
   mark `touse'      
   markout `touse' `depvar' `mainlst' `cntrlst' `oweight' `crdlst' 
   if ("`if'" ~= ""){
      capture drop _if
      generate     _if = 0
      replace      _if = 1 if `if'
      replace   `touse' = `touse' * _if
      drop _if
   } 
   gsort -`touse'
} /*quietly */


* ** CALCULATE WEIGHT
tempvar sum min max norm number
quietly egen            `sum'    = sum(`oweight')             if `touse'
quietly egen            `min'    = min(`oweight')             if `touse'
quietly egen            `max'    = max(`oweight')             if `touse'
quietly egen            `number' = count(`oweight')           if `touse'
quietly generate double `norm'   = `number' * `oweight'/`sum' if `touse'  /* premultiply by `number' so that aggregate weight is unchanged from non-weighted version */
                 local   ratio   = `max'[1]/`min'[1]


*! ** OPTIONAL 1ST STAGE REGRESSIONS **
*     regressing dependent variable and all `mainlst' variables' off of all `cntrlst' variables (cuts execution time; less display )

* need to preserve values of dependent variable and `mainlist' variables, as they will be replaced prior to 
* sending to x_ols_modified in case that first stage regression are run (in order to get x_ols_modified display correct)


quietly{regress `depvar' `mainlst' `cntrlst' [aweight=`oweight'] if `touse' }



if ("`stage1'"=="yes"){ /*regress dependent variable "off" of control variables and a constant (no need to explicitly add consant) */
     local curvar "`depvar'"
     tempvar resid0
     quietly{
         regress `curvar' `cntrlst' if `touse' [aweight=`norm']   
         predict `resid0'     if `touse',residuals   /* residuals stored in `resid_depvar' */ 
         tempvar   deptemp
         generate `deptemp' = `depvar'
         replace  `depvar' = `resid0'
         } 
    
     * now regress `mainlst' variables off of control variables (no need to explicitly add constant as 'regress' does so)
     local topass ""         /* initiate a string macro that will hold names of rhs variables to pass to x_ols_modified */
     local j = 1
     while `j' <= (`nmain') { 
        local curvar : word `j' of `mainlst'
        quietly{
        regress  `curvar' `cntrlst' if `touse' [aweight=`norm']    
        tempvar   resid`j'
        predict  `resid`j''         if `touse',residuals
    	  tempvar   main`j'
   	    generate `main`j'' = `curvar'      /* so that we can restore original values to mainlist variables */
        replace  `curvar' = `resid`j''     /* variables with original names will hold residual values to get display in x_ols_modified correct */
   	    local topass "`topass' `curvar'"   /* this iteratively builds the correct string to call with x_ols_modified */
        } /* quietly regressions to get residuals off of control list variables */
        local j = `j' + 1 } 

  di "preliminary regressions off of passive control variables complete"
  di ""
} /* (`stage1'"=="yes") */ 
else{
  di "no preliminary regressions"
  di ""
    } /* (`stage1'"=="no") */ 


*! *** RUN 'x_ols_modified' ***
if ("`stage1'"=="yes"){

     if("`suppress'"=="suppress"){
            quietly{x_ols_modified `depvar' `topass' (`crdlst'), to_use(`touse') sptlcoef(`sptlcoef') cut(`cut')  nweight(`norm') nocons cntrlst(`cntrlst') if(`if')}
         }
     else{
                    x_ols_modified `depvar' `topass' (`crdlst'), to_use(`touse') sptlcoef(`sptlcoef') cut(`cut')  nweight(`norm') nocons cntrlst(`cntrlst') if(`if')
         }
    } /* ("`stage1'"=="yes") */
else{
     if("`suppress'"=="suppress"){
            quietly{x_ols_modified `depvar' `mainlst' _intrcpt (`crdlst') , to_use(`touse') sptlcoef(`sptlcoef') cut(`cut')  nweight(`norm') nocons if(`if')}
         }
     else{
                    x_ols_modified `depvar' `mainlst' _intrcpt (`crdlst') , to_use(`touse') sptlcoef(`sptlcoef') cut(`cut')  nweight(`norm') nocons if(`if')
         }
     } /* ("`stage1'"=="no") */

*! Restore mainlist variables to their input values (in case when stage 1 regressions were run)
if ("`stage1'"=="yes"){
   quietly{replace `depvar' = `deptemp'}
   local j = 1
   while `j' <= (`nmain') { 
   local curvar : word `j' of `mainlst'
   quietly{replace `curvar' = `main`j''}
   local j = `j' + 1 } /* iterate over j */
   }   

*! *** SETUP OUTPUT MATRIX TO HOLD RESULTS OF SPATIALLY ROBUST REGRESSION ***
** Construct rownames for output matrix
local rname "________  sptlcoef cut_in_km w_ratio ________ "  /* 5 rows above args and args_wc*/
local j = 1           
   while `j' <= (`nmain'){  /* mainlst does not include depvar or constant*/
   local curvar : word `j' of `mainlst'
   local rname "`rname' `curvar':_b `curvar':se "
   local j = `j' + 1 
}
if("`stage1'"=="no"){/* additional entry for constant */
  local rname "`rname' _cons:_b _cons:se ________ N R_Sqrd AdjRSqrd SumSqrdResid RHS_Var" /*6 rows above `nmain'; 4 rows above `args_wc' */};
else{
  local rname "`rname' ________ N R_Sqrd AdjRSqrd SumSqrdResid RHS_Var" /* 4 rows above both `nmain' and `args_wc' */}
 
local rows = 2 * `args_wc' + `adjust'

tempname outpt
matrix define  `outpt' = J(`rows',1,0)
matrix colname `outpt' = `depvar' 
matrix rowname `outpt' = `rname' 
matrix `outpt'[2,1] = `sptlcoef'  /* coefficient governing convexity of spatial decline */
matrix `outpt'[3,1] = `cut'       /* distance beyond which zero spatial correlation */
matrix `outpt'[4,1] = `ratio'     /* ratio of highest weighting to lowest weighting */ 

* for placing results in a column format:
local r1 = `frstcf'
local r2 = `r1' + 1

* Rows for all variables in main variable list plus constant:
local j = 1
while `j' <= (`args_wc'){
  matrix `outpt'[`r1',1] = ${beta`j'}
  matrix `outpt'[`r2',1] = ${se2`j'}
  local r1 = `r1' + 2
  local r2 = `r2' + 2
  local j  =  `j' + 1
}

* Number of Observations and Adjusted R-squared (results are from straight OLS)
local r1 = `r1' + 1  /* N (adjust row to take account of offset row ) */
local r2 = `r1' + 1  /* R^2  */
local r3 = `r2' + 1  /* Adj R^2 */
local r4 = `r3' + 1  /* e'e  */
local r5 = `r4' + 1  /* # independent variables  */

quietly{regress `depvar' `mainlst' `cntrlst' [aweight=`oweight'] if `touse' }

matrix `outpt'[`r1',1] = _result(1)
matrix `outpt'[`r2',1] = _result(7)
matrix `outpt'[`r3',1] = _result(8)
matrix `outpt'[`r4',1] = _result(4)
matrix `outpt'[`r5',1] = _result(3)

** Display Output
di ""
di "Method:              `method' "
if(`sptlcoef'>=99){
di "Spatial Weight:      1 if dist<=`cut'; 0 otherwise"}
else{
di "Spatial Weight:      1 - (dist/`cut')^`sptlcoef'"}
if("`obsweight'"==""){
di "Obs Weight:          unweighted"
}
else{
di "Obs Weight:          `obsweight'"
}
di "Condition:           `if'"

if ("`stage1'"=="yes" & `ncntrl'<=`ncntmax'){
    if `ncntrl'==1{
    di "Additional Controls: `ncntrl' var:  `cntrlst'"}
    else{
    di "Additional Controls: `ncntrl' vars: `cntrlst'"}    
    }
else if ("`stage1'"=="yes"){
    di "Additional Controls: `ncntrl' vars"}
else{
    di "Additional Controls: none"}

matrix list `outpt', noheader format(%9.0g)

* capture mat drop regout  /* if you want to carry the results...     */
* mat regout=`outpt'       /* over to another program, erase the "*"  */
                           /* at the beginning of two lines above     */

capture drop _intrcpt
capture drop _wght

end 
* ^^^^^^^^^^^^^^^^^^^^^^^^^ 'reg_conley' ^^^^^^^^^^^^^^^^^^^^^^^^^ }



* vvvvvvvvvvvvvvvvvvvvvvv 'x_ols_modified' vvvvvvvvvvvvvvvvvvvvvvv {
*! "inner" program

*  Spatial estimator based on:									                                                                                   
* 														                                                                                                      
* 	1. Conley, Timothy G. (1999). "GMM Estimation with Cross Sectional Dependence."	                                               
* 	   Journal of Econometrics, 92, 1 (September), pp. 1-45
*  
*   2. Conley, Timothy G. (2008). "Spatial Econometrics." The New Palgrave Dictionary of Economics,
*       2nd Edition. Steven N. Durlauf and Lawrence E. Blume, eds., Palgrave Macmillan.
* 
*   3. Dube, Jean-Pierre (1997). Stata v5.0 Code for OLS Estimation with Spatial Standard Errors. (Code File). 
*      Available at http://economics.uwo.ca/people/conley_docs/code_to_download_gmm.html              		                                                     
* 														                                                                                                     
* 	4. Rappaport, Jordan (2007). "Moving to Nice Weather." Regional Science and                                                    
*      Urban Economics 37, 3 (May), pp. 375-398										                                                                 
* 														                                                                                                     
*  A revised version of Dube(1997) that does not substantively differ, Stata v6.0 Code for OLS Estimation with Location-Based Dependence,
*  is available at http://economics.uwo.ca/people/conley_docs/code_to_download_gmm.html

*  Dube's website also includes stata code for instrumental variable estimation with location-based dependence.
*  Solomon Hsiang has posted code that implements the Conley adjustment for panel data ('ols_spatial_HAC.ado')

 	
*  'x_ols_modified' is an inner program called by outer program 'reg_conley' in this file above
*   Some of the code is modified from 'x_ols" (Dube, 1997; Stata 5 version, October 11, 1997). 
* 
*  Spatial decline function is based on Euclidean distance, calculated using a great circle approximation between specified points. 
*  Econometric theory described in Conley (1999) and implementation in Dube (1999) use decline based on the product of displacement 
*  in each of two dimensions. 

capture program drop x_ols_modified  
program define x_ols_modified, sortpreserve
version 5.0

* called from 'reg_conley' with one of two forms:           
* x_ols_modified `depvar' `topass'           (`crdlst') , to_use(`touse') sptlcoef(`sptlcoef') cut(`cut')  nweight(`norm') nocons cntrlst(`cntrlst') if(`if')
* x_ols_modified `depvar' `mainlst' _intrcpt (`crdlst') , to_use(`touse') sptlcoef(`sptlcoef') cut(`cut')  nweight(`norm') nocons if(`if')
 


#delimit ;				

local input "`*'";
local varlist "required existing";
local options "to_use(string) sptlcoef(real 1) cut(real -1)  crdlst(string) if(string) nweight(string) hascons nocons cntrlst(string)";
* local weight "aweight fweight pweight iweight";
parse "`*'";   /* High Level Parse  */
parse "`*'", parse(" )([,");


*! *** GET NUMBER OF ARGUMENTS ***;
local j = 1;
while (("``j''" ~= "") & ("``j''" ~= "(" ) & ("``j''" ~= "[" ) & ("``j''" ~= ",")) {local j =`j'+1};
local args = `j'-2;     /* j=1 is LHS variable; the last j is the increment that caused loop exit */
local args_wc = `args'; /* as passed from reg_conley, either has explicit constant or already included in first stage */
local xreg =`args_wc';


tempvar Y;
quietly{generate double `Y'=`1'};			/*get dep variable*/
local depend : word 1 of `varlist';
local b=1;
local a=2;

quietly{;
  while `b'<=`args_wc'{;
	  tempvar X`b';
	  local ind`b' : word `a' of `varlist';
	  generate double `X`b''= ``a'';
	  local a=`a'+1;
	  local b=`b'+1;
	};			/*get indep var(s)...rest of list*/ 
};

local j = `j'+1;
local k = `j';
while (("``j''" ~= "") & ("``j''" ~= ")" ) & ("``j''" ~= "[" ) & ("``j''" ~= ",")) {local j =`j'+1};
local coord = `j'-`k';     /* the last j is the increment that caused loop exit */
if `coord'~=2{;
	di in red "exactly two coordinate variables must be specified";
	exit 198};
tempvar lat long;
local k=`j'-2;
quietly{generate double `lat'=``k''};
local k=`j'-1;
quietly{generate double `long'=``k''};

*! CHECK THAT CUTOFFS ARE SPECIFIED ;
if `cut'<1{;
	if `cut'==-1{;
		di in red "option cut() required!!!";
		exit 198};
	di in red "cut(`cut') is invalid";
	exit 198};	
 
 
*! DEALING WITH MISSING OBSERVATIONS;
*  Missing observations are identified in reg_conley and passed into x_ols_modified;
gsort -`to_use';

/*NOW I RUN THE REGRESSION AND COMPUTE THE COV MATRIX*/

quietly{;			/*so that steps are not printed on screen*/
	/*(1) RUN REGRESSION*/
	/*W connotes an N-by-N matrix with elements of `nweight' along its diagonal*/
	tempname XWX XWX_N invXWX invN;  
	
	if `xreg'==1 {; /*if only 1 RHS variable, run these regs; else, if more than 1 RHS var, run below regs; seems that constant is being treated as a RHS variable therefore the else routine runs for most*/
		di "`varlist'";
		regress `Y' `X1'           [aweight=`nweight'] if `to_use' ,         noconstant;  /* calculate and store ols standard error for display comparison   */
*		local se_ols=_se[`X1'];                                                       
    local z=1;
		while `z'<=`xreg'{;
		  local se_ols`z'=_se[`X`z''];														
		local z=`z'+1};
			                                                                            
		regress `Y' `X1'           [aweight=`nweight'] if `to_use' , robust  noconstant /* regress with robust option (leaves White standard errors in memory) */
  }; /* `xreg'==1 */
			
	else{;  /* `xreg'>1*/
		regress `Y' `X1'-`X`xreg'' [aweight=`nweight'] if `to_use' ,         noconstant; /* calculate and store ols standard error for display comparison   */
    local z=1;
		while `z'<=`xreg'{;
		  local se_ols`z'=_se[`X`z''];														
		local z=`z'+1};
												
		regress `Y' `X1'-`X`xreg'' [aweight=`nweight'] if `to_use' , robust  noconstant   /* regress with robust option (leaves White standard errors in memory) */
	};


	tempvar epsilon wepsilon;
	predict `epsilon' if `to_use',residuals;	/* unweighted OLS residuals */

	* generate double `wepsilon' = sqrt(`nweight') * `epsilon' ; /* weighted OLS residuals */
	local _n = _result(1);
	scalar `invN'=1/`_n';     /* note weights are normalized so that they sum to _n */
	if `xreg'==1 {;
		mat accum `XWX'=`X1' [aweight=`nweight'] if `to_use',noconstant};
	else{;
		mat accum `XWX'=`X1'-`X`xreg'' [aweight=`nweight'] if `to_use',noconstant};
	mat `XWX_N'=`XWX'*`invN';    /* creates X'WX/N   */
	mat `invXWX'=inv(`XWX_N');	/* creates (X'WX/N)^(-1)*/

	/*(2) COMPUTE CORRECTED COVARIANCE MATRIX*/
	tempname XUWWUX XUWWUX1 XUWWUX2 XUWWUXt;
	tempvar XUWUk;
	tempvar window;
	tempvar dist;
	mat `XUWWUX'=J(`xreg',`xreg',0);
	generate double `XUWUk'=0;
	generate double `window'=1;			
	generate double `dist'=0;
	* convert coordinates to radians;
	replace `long' = `long'*(_pi/180);
	replace `lat' = `lat'*(_pi/180);

	local i=1;
	while `to_use'[`i']==1{			/*loop through non-missing observations*/
		replace `window'=0;
		replace `dist' = acos((sin(`lat')*sin(`lat'[`i'])+
			cos(`lat')*cos(`lat'[`i'])*cos(`long'[`i']-`long')))*6378.137;
		
		** vvvvvv for testing purposes vvvvvv;
		* these come within 2% of standard errors calculated by stata built-in routines ;
		* I believe the difference may be due to accumulated rounding errors ;
		* replace `window'=1 if ID_FIPST[`i']==ID_FIPST; /* analogous to cluster(ID_FIPST) */ 
		* replace `window'= 1 if (`dist'<0.00014);       /* analogous to "robust" */
		** ^^^^^^ for testing purposes ^^^^^^;
		
	  /* vvvvvv DECLINING GEOGAPHIC DISTANCE WEIGHTING WITH USER SET CONVEXITY vvvv               */
	  /* if sptlcoef >= 99, use non-continuous decline; this is similar to 'cluster'              */
	  /* option in Stata, except that only partial overlap of observations in covariance          */
	  /* matrix. Allows for obs 1 and 2 may to have correlation and 2 and 3 to have correlation   */
	  /* but assumes that 1 and 3 are uncorrelated                                                */	  
	  if(`sptlcoef'>=99){;
	  replace `window'=1                           if (`dist'<=`cut'); 
	  };
	  else{;
		replace `window'=1-(`dist'/`cut')^`sptlcoef' if (`dist'<`cut');   
    };
		/* first deriv = (-sptlcoef/(cut^sptlcoef))*dist^(sptlcoef -1)                */
		/* secnd deriv = -(sptlcoef-1)(sptlcoef)/(cut^sptlcoef) * dist^(sptlcoef-2)   */
		/* so sptlcoef = 1 -> linear decline                                          */
		/* so sptlcoef > 1 -> neg second deriv, fall off increasin with distance      */
		/* so sptlcoef < 1 -> pos second deriv, fall off slowing with distance        */

		
		capture mat drop `XUWWUX2';  /* this is the k-by-k matrix into which error contributions from each observation will be accumulated */
		local k=1;
		while `k'<=`xreg'{;

      *!!  two equivalent ways to handle weighting  !!** 
			* version 1 weighting (implemented);
			*      note:  intuitively it seemed to me that would want to use sqrt(`nweight') rather than linear;
			*             but comparison to pre-multiplied optimal estimator shows the weighting that follows is correct;
			replace `XUWUk'=(`X`k''[`i'])*(`nweight'[`i'])*(`epsilon'[`i'])*`epsilon'*(`nweight')*`window'; /*creates N-by-1 vector */
			mat vecaccum `XUWWUX1'=`XUWUk' `X1'-`X`xreg'' if `to_use', noconstant; /* creates 1-by-k vector*/

			* version 2 weighting (not implemented); 
			* replace `XUWUk'=(`X`k''[`i'])*sqrt(`nweight'[`i'])*(`epsilon'[`i'])*`epsilon'*`window'; /*creates N-by-1 vector */
			* mat vecaccum `XUWWUX1'=`XUWUk' `X1'-`X`xreg'' [aweight=(`nweight')] if `to_use', noconstant; /* creates 1-by-k vector*/
		
		  mat `XUWWUX2'=`XUWWUX2' \ `XUWWUX1';  /*stacks latter below former to eventually get a k-by-k matrix */

   	local k=`k'+1};

		mat `XUWWUXt'=`XUWWUX2'';           /* transpose */
		mat `XUWWUX1'=`XUWWUX2'+`XUWWUXt';  /* adding transpose to original: still k-by-k */
		scalar fix=.5;		                  /*to correct for double-counting*/
		mat `XUWWUX1'=`XUWWUX1'*fix;
		mat `XUWWUX'=`XUWWUX'+`XUWWUX1';    /*add current observation k-by-k into all observation k-by-k matrix */

 local i=`i'+1}; /*loop through all non-missing observations*/

 mat `XUWWUX'=`XUWWUX'*`invN'; 

};					/*end quietly command*/


tempname V VV;
mat `V'=`invXWX'*`XUWWUX';
mat `VV'=`V'*`invXWX';

matrix cov_dep=`VV'*`invN';		/*corrected covariance matrix*/


/*THIS PART CREATES AND PRINTS THE OUTPUT TABLE IN STATA*/
local z=1;
local v=`a';
di _newline(2) "Results for GLS with standard errors robust to spatially-correlated errors:" ;
* di _newline	_col(35)	" number of observations=  "  _result(1);
* di _newline(2) ; 
di "Dependent Variable:  `depend'";
di "Additional Controls: `cntrlst'";
di "If condition:        `if'";
di _newline
"variable" _col(34) "ols estimates" _col(50) "ols s.e." _col(63) "White s.e." _col(76) 
	"s.e. robust to spatial correlation";
di 
"--------" _col(34) "-------------" _col(50) "----------" _col(63) "----------" _col(76) 
	"----------------------------------";

/* here's where actual assignment of coefficients and standard errors occurs: need to get it for constant */
while `z'<=`xreg'{;
  local zz=`z'+1;
	global beta`z'=_b[`X`z''];
	local se`z'=_se[`X`z''];
	local  se1`z'=cov_dep[`z',`z'];
	global  se2`z'=sqrt(`se1`z'');
  local curvar : word `zz' of `varlist';
	di "`curvar'" _col(34)  ${beta`z'}  _col(50) `se_ols`z''  _col(63)  `se`z'' _col(76) ${se2`z'};
*	di "`ind`z''" _col(34)  ${beta`z'}  _col(50) `se_ols`z''  _col(63)  `se`z'' _col(76) ${se2`z'};
local z=`z'+1};
  di "_____________________________________________________________________________________________________";

end;
* ^^^^^^^^^^^^^^^^^^^^^^^ 'x_ols_modified' ^^^^^^^^^^^^^^^^^^^^^^^ }

exit;


